/*
 * Simulator of MCS51 (pobj.h)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#ifndef PObjHeader
#define PObjHeader

#include "ddconfig.h"

#include "pobjt.h"

typedef class TObject		*PObject;
typedef class TCollection	*PCollection;
typedef class TSortedCollection *PSortedCollection;
typedef class TStringCollection *PStringCollection;
typedef class TUnsortedStrings	*PUnsortedStrings;

/*									    #
  ==========================================================================#
								    TObject #
  ==========================================================================#
									    #
*/

class TObject
{
public:
          TObject(void);
  virtual ~TObject(void);

  virtual int Init(void);
};


/*									    #
  ==========================================================================#
								Tcollection #
  ==========================================================================#
									    #
*/

class TCollection: public TObject
{
protected:
  void		   **Items;
  TIndex	   Count;
  TIndex	   Limit;
  TIndex	   Delta;

public:
		   TCollection(TIndex ALimit, TIndex ADelta);
  virtual      	   ~TCollection(void);

	  void	   *At(TIndex index);
  virtual TIndex   IndexOf(void *Item);
	  int	   GetCount(void);

	  void	   Pack(void);
  virtual void	   SetLimit(TIndex ALimit);

	  void	   AtFree(TIndex Index);
	  void	   AtDelete(TIndex Index);
	  void	   Delete(void *Item);
	  void	   DeleteAll(void);
//	  void	   Free(void *Item);
//	  void	   FreeAll(void);

	  void	   AtInsert(TIndex Index, void *Item);
	  void	   AtPut(TIndex Index, void *Item);
  virtual TIndex   Insert(void *Item);

	  void	   *FirstThat(TTestFunc Test, void *Arg);
	  void	   *LastThat(TTestFunc Test, void *Arg);
	  void	   ForEach(TAppFunc Action, void *Arg);

	  void	   Error(TIndex Code, TIndex Info);
private:
  virtual void	   FreeItem(void *Item);
};


/*									    #
  ==========================================================================#
							  TSortedCollection #
  ==========================================================================#
									    #
*/

class TSortedCollection: public TCollection
{
public:
  bool		   Duplicates;

          TSortedCollection(TIndex ALimit, TIndex ADelta);
  virtual ~TSortedCollection(void);
  
  virtual bool	   Search(void *Key, TIndex& Index);
  virtual TIndex   IndexOf(void *Item);
  virtual TIndex   Insert(void *Item);
  virtual void	   *KeyOf(void *Item);
private:
  virtual int	   Compare(void *Key1, void *Key2)= 0;
};


/*									    #
  ==========================================================================#
							  TStringCollection #
  ==========================================================================#
									    #
*/

class TStringCollection: public TSortedCollection
{
public:
          TStringCollection(TIndex ALimit, TIndex ADelta);
  virtual ~TStringCollection(void);
  
private:
  virtual int	   Compare(void *Key1, void *Key2);
  virtual void	   FreeItem(void *Item);
};


/*									    #
  ==========================================================================#
							   TUnsortedStrings #
  ==========================================================================#
									    #
*/

class TUnsortedStrings: public TStringCollection
{
public:
          TUnsortedStrings(TIndex ALimit, TIndex ADelta);
  virtual ~TUnsortedStrings(void);
  
private:
  virtual int	   Compare(void *Key1, void *Key2);
  virtual bool	   Search(void *Key, TIndex &Index);
};


#endif

/* End of pobj.h */
